@@ -19,6 +19,7 @@
  * Copyright (C) 2006 - 2008 Novell, Inc.
  */
 
+#include "config.h"
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <netinet/in.h>
@@ -34,7 +35,6 @@
 #include <linux/if.h>
 #include <errno.h>
 
-#define G_UDEV_API_IS_SUBJECT_TO_CHANGE
 #include <gudev/gudev.h>
 
 #include <netlink/route/addr.h>
@@ -358,7 +358,7 @@ _update_s390_subchannels (NMDeviceEthernet *self)
 				priv->subchan3 = g_path_get_basename (buf);
 		} else {
 			nm_log_warn (LOGD_DEVICE | LOGD_HW,
-			             "(%s): failed to read cdev link '%s': %s",
+			             "(%s): failed to read cdev link '%s': %d",
 			             iface, cdev_path, errno);
 		}
 		g_free (cdev_path);
@@ -668,8 +668,7 @@ _set_hw_addr (NMDeviceEthernet *self, const guint8 *addr, const char *detail)
 
 	/* Do nothing if current MAC is same */
 	if (!memcmp (&priv->hw_addr, addr, ETH_ALEN)) {
-		nm_log_dbg (LOGD_DEVICE | LOGD_ETHER, "(%s): no MAC address change needed",
-		            iface, detail, mac_str);
+		nm_log_dbg (LOGD_DEVICE | LOGD_ETHER, "(%s): no MAC address change needed", iface);
 		g_free (mac_str);
 		return TRUE;
 	}
@@ -2158,7 +2157,7 @@ supports_mii_carrier_detect (NMDeviceEthernet *self)
 	fd = socket (PF_INET, SOCK_DGRAM, 0);
 	if (fd < 0) {
 		nm_log_err (LOGD_HW, "couldn't open control socket.");
-		return 0;
+		return FALSE;
 	}
 
 	memset (&ifr, 0, sizeof (struct ifreq));
