@@ -527,7 +527,7 @@ request_add_agent (Request *req,
 	/* If the request should filter agents by UID, do that now */
 	if (req->filter_by_uid && (agent_uid != req->uid_filter)) {
 		nm_log_dbg (LOGD_AGENTS, "(%s) agent ignored for secrets request %p/%s "
-		            "(uid %ld not required %ld)",
+		            "(uid %d not required %ld)",
 				    nm_secret_agent_get_description (agent),
 				    req, req->setting_name, agent_uid, req->uid_filter);
 		return;
@@ -729,9 +729,10 @@ get_agent_request_secrets (Request *req, gboolean include_system_secrets)
 
 	tmp = nm_connection_duplicate (req->connection);
 	nm_connection_clear_secrets (tmp);
-	if (include_system_secrets)
-		nm_connection_update_secrets (tmp, req->setting_name, req->existing_secrets, NULL);
-	else {
+	if (include_system_secrets) {
+		if (req->existing_secrets)
+			nm_connection_update_secrets (tmp, req->setting_name, req->existing_secrets, NULL);
+	} else {
 		/* Update secret flags in the temporary connection to indicate that
 		 * the system secrets we're not sending to the agent aren't required,
 		 * so the agent can properly validate UI controls and such.
