@@ -24,7 +24,6 @@
 #include "nm-active-connection.h"
 #include "nm-vpn-connection-base-glue.h"
 #include "nm-dbus-manager.h"
-#include "nm-setting-connection.h"
 
 G_DEFINE_ABSTRACT_TYPE (NMVpnConnectionBase, nm_vpn_connection_base, G_TYPE_OBJECT)
 
@@ -37,7 +36,6 @@ typedef struct {
 
 	NMConnection *connection;
 	char *ac_path;
-	char *specific_object;
 	gboolean is_default;
 	gboolean is_default6;
 	NMActiveConnectionState state;
@@ -45,9 +43,7 @@ typedef struct {
 
 enum {
 	PROP_0,
-	PROP_SERVICE_NAME,
 	PROP_CONNECTION,
-	PROP_UUID,
 	PROP_SPECIFIC_OBJECT,
 	PROP_DEVICES,
 	PROP_STATE,
@@ -94,16 +90,9 @@ nm_vpn_connection_base_get_ac_path (NMVpnConnectionBase *self)
 	return NM_VPN_CONNECTION_BASE_GET_PRIVATE (self)->ac_path;
 }
 
-const char *
-nm_vpn_connection_base_get_specific_object (NMVpnConnectionBase *self)
-{
-	return NM_VPN_CONNECTION_BASE_GET_PRIVATE (self)->specific_object;
-}
-
 void
 nm_vpn_connection_base_export (NMVpnConnectionBase *self,
-                               NMConnection *connection,
-                               const char *specific_object)
+                               NMConnection *connection)
 {
 	NMVpnConnectionBasePrivate *priv = NM_VPN_CONNECTION_BASE_GET_PRIVATE (self);
 	NMDBusManager *dbus_mgr;
@@ -111,7 +100,6 @@ nm_vpn_connection_base_export (NMVpnConnectionBase *self,
 	g_return_if_fail (priv->connection == NULL);
 
 	priv->connection = g_object_ref (connection);
-	priv->specific_object = g_strdup (specific_object);
 
 	dbus_mgr = nm_dbus_manager_get ();
 	dbus_g_connection_register_g_object (nm_dbus_manager_get_connection (dbus_mgr),
@@ -139,7 +127,6 @@ dispose (GObject *object)
 		priv->disposed = TRUE;
 
 		g_free (priv->ac_path);
-		g_free (priv->specific_object);
 		g_object_unref (priv->connection);
 	}
 
@@ -151,22 +138,13 @@ get_property (GObject *object, guint prop_id,
 			  GValue *value, GParamSpec *pspec)
 {
 	NMVpnConnectionBasePrivate *priv = NM_VPN_CONNECTION_BASE_GET_PRIVATE (object);
-	NMSettingConnection *s_con;
 
 	switch (prop_id) {
-	case PROP_SERVICE_NAME:
-		nm_active_connection_scope_to_value (priv->connection, value);
-		break;
 	case PROP_CONNECTION:
 		g_value_set_boxed (value, nm_connection_get_path (priv->connection));
 		break;
-	case PROP_UUID:
-		s_con = (NMSettingConnection *) nm_connection_get_setting (priv->connection, NM_TYPE_SETTING_CONNECTION);
-		g_assert (s_con);
-		g_value_set_string (value, nm_setting_connection_get_uuid (s_con));
-		break;
 	case PROP_SPECIFIC_OBJECT:
-		g_value_set_boxed (value, priv->specific_object);
+		g_value_set_boxed (value, priv->ac_path);
 		break;
 	case PROP_DEVICES:
 		g_value_take_boxed (value, g_ptr_array_new ());
@@ -202,9 +180,7 @@ nm_vpn_connection_base_class_init (NMVpnConnectionBaseClass *vpn_class)
 
 	/* properties */
     nm_active_connection_install_properties (object_class,
-                                             PROP_SERVICE_NAME,
                                              PROP_CONNECTION,
-                                             PROP_UUID,
                                              PROP_SPECIFIC_OBJECT,
                                              PROP_DEVICES,
                                              PROP_STATE,
